/*
 *      Simple User Authentication System -- PAM module
 *
 *      (c) 1999 Martin Hinner, <martin@tdp.cz>
 *
 *      This software may be freely distributed and used according to the terms
 *      of the GNU General Public License. See file COPYING in any of the GNU packages.
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <syslog.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/types.h>

#define PAM_SM_AUTH

#define PAM_SM_AUTH

#include <security/pam_modules.h>
#include <security/_pam_macros.h>

static char use_group[BUFSIZ];

static int
suas_greply(int sock, char *buf)
{
  int l = read(sock, buf, 256);
  if (l <= 2)
    return 0;
  buf[l-1] = 0;
  return (buf[0] == '+');
}

static int converse(pam_handle_t *pamh, int nargs
                    , struct pam_message **message
                    , struct pam_response **response)
{
    int retval;
    struct pam_conv *conv;

    retval = pam_get_item( pamh, PAM_CONV, (const void **) &conv ) ;
    if ( retval == PAM_SUCCESS ) {

        retval = conv->conv(nargs, ( const struct pam_message ** ) message
                            , response, conv->appdata_ptr);

        if (retval != PAM_SUCCESS) {
            syslog(LOG_DEBUG, "conversation failure [%s]"
                     , pam_strerror(pamh, retval));
        }

    } else {
        syslog(LOG_ERR, "couldn't obtain coversation function [%s]"
                 , pam_strerror(pamh, retval));
    }

    return retval;                  /* propagate error status */
}


PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
			,const char **argv)
{
     const char *username;
     int sock, r;
     struct sockaddr_un sa = { AF_UNIX, "/var/run/suamaster" };
     char mbuf[256];
     char *password;
     struct pam_message msg[1], *mesg[1];
     struct pam_response *resp=NULL;
     const char *token;
     char *prompt=NULL;
     int i=0;
     int retval;


     openlog("pam_suas",LOG_PID,LOG_AUTH);
 
     if ((pam_get_item( pamh, PAM_USER,(void*)&username) != PAM_SUCCESS) || (!username)) {
	closelog();
        return PAM_SERVICE_ERR;
     }

     if ((pam_get_item( pamh, PAM_AUTHTOK,(void*)&password) != PAM_SUCCESS) || (!password)) {

       mesg[i] = &msg[i];
       msg[i].msg_style = PAM_PROMPT_ECHO_OFF;
       msg[i].msg = "Password:";

       retval = converse(pamh, ++i, mesg, &resp);
       _pam_overwrite(prompt);
       _pam_drop(prompt);
       if (retval != PAM_SUCCESS) {
           if (resp != NULL)
               _pam_drop_reply(resp,i);
           closelog();
           return PAM_AUTHINFO_UNAVAIL;
       }

	password = resp->resp;
     }

     (void) pam_set_item(pamh, PAM_AUTHTOK, password);

  sock = socket(AF_UNIX, SOCK_STREAM, 0);
  if (sock < 0)
    {
    grr:
      syslog(LOG_ERR, "Unable to connect to authentication server, login rejected.");
      closelog();
      return PAM_IGNORE;
    }
  if (connect(sock, (struct sockaddr *) &sa, sizeof(sa)) < 0)
    {
    grr2:
      close(sock);
      goto grr;
    }
  if (!suas_greply(sock, mbuf))
    goto grr2;
  write(sock, mbuf, sprintf(mbuf, "%s\001%s\n", username, password));
  r = suas_greply(sock, mbuf);
  if (!r)           /* Don't close the socket in case the auth succe */
    close(sock);
  closelog();
  return r?PAM_SUCCESS:PAM_PERM_DENIED;
}

PAM_EXTERN
int pam_sm_setcred(pam_handle_t *pamh,int flags,int argc
		   ,const char **argv)
{
     return PAM_SUCCESS;
}
