/*
 *	Simple User Authentication System -- Global Declarations
 *
 *	(c) 1996 -- 1998 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU General Public License. See file COPYING in any of the GNU packages.
 */

/* Version number */

#define SUA_VERSION "1.2"
#define SUA_YEAR "1996--1998"

/* Types and so on */

#define NO_RET __attribute__ ((noreturn))

typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned int ulg;

/* Configuration */

#define NFS_HACK			/* suad: Talk with local NFS server */
#undef USE_UNIX_SOCK_AUTH		/* suad: Use SCM_CREDENTIALS for unix socket authentication (Linux 2.1 only) */
#define USE_LOCAL_PASSWORDS		/* suad: When user has encrypted password set to "$local",
					 *       try to authenticate him using /etc/shadow. */

#define LOCAL_AUTH			/* suamaster: Try to authenticate users locally */

#ifndef __GLIBC__
#define PW_ENCRYPT 			/* suamaster: Use pw_encrypt() for local passwords */
#endif

#define SUA_SESSION_BIN_KEY_SIZE 15	/* These parameters are >magic< ! */
#define SUA_SESSION_KEY_SIZE 20
#define SUA_USER_BIN_KEY_SIZE 15
#define SUA_USER_KEY_SIZE 64
#define SUA_USER_BIN_SALT_SIZE 15
#define SUA_USER_SALT_SIZE 64
#define SUA_NET_BIN_KEY_SIZE 15
#define SUA_NET_KEY_SIZE 20

#define SUA_SERVER_PORT 997
#define SUA_CLIENT_MASTER_PORT 998

#define SUA_USER_LIST_FILE "/etc/suas/userlist"
#define SUA_USER_LIST_BAK_FILE "/etc/suas/userlist.old"
#define SUA_CLIENT_LIST_FILE "/etc/suas/clientlist"
#define SUA_LOG_FILE "/var/log/sualog"
#define SUA_PASSWD "/etc/suas/passwd"
#define SUA_MASTER_SOCKET "/var/run/suamaster"
#define SUA_DAEMON_SOCKET "/var/run/suad"

#define SUA_TIMEOUT 300			/* Connection timeout */
#define SUA_RETRY_TIMEOUT 1		/* Unsuccessful authentication */
#define SUA_REFRESH 30			/* suamaster: interval between pings */

#define SUA_STANDALONE 1                /* suamaster: standalone */

/* sualib.c */

extern char program_name[];

void die(char *, ...) NO_RET;
void *xmalloc(ulg);
void encrypt_first(char *, char *, char *);
void encrypt_local(char *, char *, char *);
void encrypt_net(char *, char *, char *);
void encode_base64(byte *, char *, int);
void generate_random(char *, ulg);
void setsig(int sig, void (*func)(int));

/* pw_encrypt */

#if defined(LOCAL_AUTH) || defined(USE_LOCAL_PASSWORDS)
#include <shadow.h>
#ifdef PW_ENCRYPT
extern char *pw_encrypt(const char *, const char *);
#else
#include <crypt.h>
#define pw_encrypt crypt
#endif
#endif
